@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
+#  Copyright (c) 1994 - 99	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -48,11 +48,11 @@
 #define DATAFILE "/usr/games/lib/mball.data"
 #endif
 
-static void InitializeMball(Widget request, Widget new);
-static void ExposeMball(Widget new, XEvent * event, Region region);
+static void InitializeMball(Widget request, Widget renew);
+static void ExposeMball(Widget renew, XEvent * event, Region region);
 static void ResizeMball(MballWidget w);
 static void DestroyMball(Widget old);
-static Boolean SetValuesMball(Widget current, Widget request, Widget new);
+static Boolean SetValuesMball(Widget current, Widget request, Widget renew);
 static void QuitMball(MballWidget w, XEvent * event, char **args, int nArgs);
 static void PracticeMball(MballWidget w, XEvent * event, char **args, int nArgs);
 static void PracticeMballMaybe(MballWidget w, XEvent * event, char **args, int nArgs);
@@ -84,13 +84,15 @@ static void MoveMballCcw(MballWidget w, XEvent * event, char **args, int nArgs);
 static void MoveMballInput(MballWidget w, int x, int y, int direction, int control);
 static void SelectMball(MballWidget w, XEvent * event, char **args, int nArgs);
 static void ReleaseMball(MballWidget w, XEvent * event, char **args, int nArgs);
-static void GetColor(MballWidget w, int wedge, int init);
+
+static void SetAllColors(MballWidget w, Boolean init);
+static void GetColor(MballWidget w, int wedge, Boolean init);
 static void MoveControlCb(MballWidget w, int wedge, int direction);
 static void CheckWedges(MballWidget w);
 static void ResetWedges(MballWidget w);
 static void ResizeWedges(MballWidget w);
-static int  SelectWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *view);
-static int  PositionWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *direction);
+static Boolean SelectWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *view);
+static Boolean PositionWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *direction);
 static void MoveNoWedges(MballWidget w);
 static void PracticeWedges(MballWidget w);
 static void RandomizeWedges(MballWidget w);
@@ -107,14 +109,15 @@ static void LetterPosition(MballWidget w, int wedge, int ring, int lengthx, int
 static void OffsetSect(MballWidget w, int wedge, int *dx, int *dy);
 static void XFillSector(Display * display, Drawable drawable, GC gc, int xo, int yo, int width1, int height1, int width2, int height2, int angle1, int angle2);
 static void XDrawSector(Display * display, Drawable drawable, GC gc, int xo, int yo, int width1, int height1, int width2, int height2, int angle1, int angle2);
+static int  int2String(char *buf, int number, int base, Boolean capital);
 
 #ifdef DEBUG
-static void 
-PrintMball(MballWidget w)
+static void PrintMball(MballWidget w);
+
 #endif
 
-	static char defaultTranslationsMball[] =
-	"<KeyPress>q: Quit()\n\
+static char defaultTranslationsMball[] =
+"<KeyPress>q: Quit()\n\
    Ctrl<KeyPress>C: Quit()\n\
    <KeyPress>KP_Divide: MoveCcw()\n\
    <KeyPress>Home: MoveTl()\n\
@@ -163,138 +166,148 @@ PrintMball(MballWidget w)
    <KeyPress>4: Wedge4()\n\
    <KeyPress>6: Wedge6()\n\
    <KeyPress>8: Wedge8()\n\
-   <KeyPress>0: Wedge12()";
+   <KeyPress>0: Wedge10()\n\
+   <KeyPress>=: Wedge12()\n\
+   <KeyPress>.: Wedge12()";
 
-	static XtActionsRec actionsListMball[] =
-	{
-		{"Quit", (XtActionProc) QuitMball},
-		{"MoveCcw", (XtActionProc) MoveMballCcw},
-		{"MoveTl", (XtActionProc) MoveMballTl},
-		{"MoveTop", (XtActionProc) MoveMballTop},
-		{"MoveTr", (XtActionProc) MoveMballTr},
-		{"MoveLeft", (XtActionProc) MoveMballLeft},
-		{"MoveCw", (XtActionProc) MoveMballCw},
-		{"MoveRight", (XtActionProc) MoveMballRight},
-		{"MoveBl", (XtActionProc) MoveMballBl},
-		{"MoveBottom", (XtActionProc) MoveMballBottom},
-		{"MoveBr", (XtActionProc) MoveMballBr},
-		{"Select", (XtActionProc) SelectMball},
-		{"Release", (XtActionProc) ReleaseMball},
-		{"Practice", (XtActionProc) PracticeMball},
-		{"PracticeMaybe", (XtActionProc) PracticeMballMaybe},
-		{"Randomize", (XtActionProc) RandomizeMball},
-		{"RandomizeMaybe", (XtActionProc) RandomizeMballMaybe},
-		{"Get", (XtActionProc) GetMball},
-		{"Write", (XtActionProc) WriteMball},
-		{"Undo", (XtActionProc) UndoMball},
-		{"Solve", (XtActionProc) SolveMball},
-		{"Increment", (XtActionProc) IncrementMball},
-		{"Decrement", (XtActionProc) DecrementMball},
-		{"Orientize", (XtActionProc) OrientizeMball},
-		{"Wedge2", (XtActionProc) Wedge2ModeMball},
-		{"Wedge4", (XtActionProc) Wedge4ModeMball},
-		{"Wedge6", (XtActionProc) Wedge6ModeMball},
-		{"Wedge8", (XtActionProc) Wedge8ModeMball},
-		{"Wedge10", (XtActionProc) Wedge10ModeMball},
-		{"Wedge12", (XtActionProc) Wedge12ModeMball}
+/* '=' is the 2nd key to the right of '0' on keyboard
+   '.' follows '0' on keypad
+ */
+
+static XtActionsRec actionsListMball[] =
+{
+	{"Quit", (XtActionProc) QuitMball},
+	{"MoveCcw", (XtActionProc) MoveMballCcw},
+	{"MoveTl", (XtActionProc) MoveMballTl},
+	{"MoveTop", (XtActionProc) MoveMballTop},
+	{"MoveTr", (XtActionProc) MoveMballTr},
+	{"MoveLeft", (XtActionProc) MoveMballLeft},
+	{"MoveCw", (XtActionProc) MoveMballCw},
+	{"MoveRight", (XtActionProc) MoveMballRight},
+	{"MoveBl", (XtActionProc) MoveMballBl},
+	{"MoveBottom", (XtActionProc) MoveMballBottom},
+	{"MoveBr", (XtActionProc) MoveMballBr},
+	{"Select", (XtActionProc) SelectMball},
+	{"Release", (XtActionProc) ReleaseMball},
+	{"Practice", (XtActionProc) PracticeMball},
+	{"PracticeMaybe", (XtActionProc) PracticeMballMaybe},
+	{"Randomize", (XtActionProc) RandomizeMball},
+	{"RandomizeMaybe", (XtActionProc) RandomizeMballMaybe},
+	{"Get", (XtActionProc) GetMball},
+	{"Write", (XtActionProc) WriteMball},
+	{"Undo", (XtActionProc) UndoMball},
+	{"Solve", (XtActionProc) SolveMball},
+	{"Increment", (XtActionProc) IncrementMball},
+	{"Decrement", (XtActionProc) DecrementMball},
+	{"Orientize", (XtActionProc) OrientizeMball},
+	{"Wedge2", (XtActionProc) Wedge2ModeMball},
+	{"Wedge4", (XtActionProc) Wedge4ModeMball},
+	{"Wedge6", (XtActionProc) Wedge6ModeMball},
+	{"Wedge8", (XtActionProc) Wedge8ModeMball},
+	{"Wedge10", (XtActionProc) Wedge10ModeMball},
+	{"Wedge12", (XtActionProc) Wedge12ModeMball}
 };
 
-	static XtResource resourcesMball[] =
-	{
-		{XtNuserName, XtCUserName, XtRString, sizeof (String),
-		 XtOffset(MballWidget, mball.username), XtRString, "nobody"},
-		{XtNwedgeColor0, XtCLabel, XtRString, sizeof (String),
-	     XtOffset(MballWidget, mball.wedgeName[0]), XtRString, "Yellow"},
-		{XtNwedgeColor1, XtCLabel, XtRString, sizeof (String),
-	       XtOffset(MballWidget, mball.wedgeName[1]), XtRString, "Blue"},
-		{XtNwedgeColor2, XtCLabel, XtRString, sizeof (String),
-		 XtOffset(MballWidget, mball.wedgeName[2]), XtRString, "Red"},
-		{XtNwedgeColor3, XtCLabel, XtRString, sizeof (String),
-	    XtOffset(MballWidget, mball.wedgeName[3]), XtRString, "Magenta"},
-		{XtNwedgeColor4, XtCLabel, XtRString, sizeof (String),
-	      XtOffset(MballWidget, mball.wedgeName[4]), XtRString, "Green"},
-		{XtNwedgeColor5, XtCLabel, XtRString, sizeof (String),
-	     XtOffset(MballWidget, mball.wedgeName[5]), XtRString, "Orange"},
-		{XtNwedgeColor6, XtCLabel, XtRString, sizeof (String),
-	       XtOffset(MballWidget, mball.wedgeName[6]), XtRString, "Cyan"},
-		{XtNwedgeColor7, XtCLabel, XtRString, sizeof (String),
-	  XtOffset(MballWidget, mball.wedgeName[7]), XtRString, "Dark Green"},
-		{XtNwedgeColor8, XtCLabel, XtRString, sizeof (String),
-	  XtOffset(MballWidget, mball.wedgeName[8]), XtRString, "Pink"},
-		{XtNwedgeColor9, XtCLabel, XtRString, sizeof (String),
-	  XtOffset(MballWidget, mball.wedgeName[9]), XtRString, "Light Gray"},
-		{XtNwedgeColor10, XtCLabel, XtRString, sizeof (String),
-	  XtOffset(MballWidget, mball.wedgeName[10]), XtRString, "Aquamarine"},
-		{XtNwedgeColor11, XtCLabel, XtRString, sizeof (String),
-	  XtOffset(MballWidget, mball.wedgeName[11]), XtRString, "Turquoise"},
-		{XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
+static XtResource resourcesMball[] =
+{
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.username), XtRString, "nobody"},
+	{XtNwedgeColor0, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[0]), XtRString, "Yellow"},
+	{XtNwedgeColor1, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[1]), XtRString, "Blue"},
+	{XtNwedgeColor2, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[2]), XtRString, "Red"},
+	{XtNwedgeColor3, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[3]), XtRString, "Magenta"},
+	{XtNwedgeColor4, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[4]), XtRString, "Green"},
+	{XtNwedgeColor5, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[5]), XtRString, "Orange"},
+	{XtNwedgeColor6, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[6]), XtRString, "Cyan"},
+	{XtNwedgeColor7, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[7]), XtRString, "Dark Green"},
+	{XtNwedgeColor8, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[8]), XtRString, "Pink"},
+	{XtNwedgeColor9, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[9]), XtRString, "Light Gray"},
+	{XtNwedgeColor10, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[10]), XtRString, "Aquamarine"},
+	{XtNwedgeColor11, XtCLabel, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.wedgeName[11]), XtRString, "Turquoise"},
+	{XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
     XtOffset(MballWidget, mball.foreground), XtRString, XtDefaultForeground},
-		{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
+	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
    XtOffset(MballWidget, mball.borderColor), XtRString, XtDefaultForeground},
-		{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
-		 XtOffset(MballWidget, core.width), XtRString, "200"},
-		{XtNheight, XtCHeight, XtRDimension, sizeof (Dimension),
-		 XtOffset(MballWidget, core.height), XtRString, "400"},
-		{XtNmono, XtCMono, XtRBoolean, sizeof (Boolean),
-		 XtOffset(MballWidget, mball.mono), XtRString, "FALSE"},
-		{XtNwedges, XtCWedges, XtRInt, sizeof (int),
-		 XtOffset(MballWidget, mball.wedges), XtRString, "8"},	/*DEFAULTWEDGES */
-		{XtNrings, XtCRings, XtRInt, sizeof (int),
-		 XtOffset(MballWidget, mball.rings), XtRString, "4"},	/*DEFAULTRINGS */
-		{XtNorient, XtCOrient, XtRBoolean, sizeof (Boolean),
-		 XtOffset(MballWidget, mball.orient), XtRString, "FALSE"},	/*DEFAULTORIENT */
-		{XtNpractice, XtCPractice, XtRBoolean, sizeof (Boolean),
-		 XtOffset(MballWidget, mball.practice), XtRString, "FALSE"},	/*DEFAULTPRACTICE */
-		{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
-		 XtOffset(MballWidget, mball.started), XtRString, "FALSE"},
-	      {XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
-	       XtOffset(MballWidget, mball.select), XtRCallback, NULL}
+	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
+	 XtOffset(MballWidget, core.width), XtRString, "200"},
+	{XtNheight, XtCHeight, XtRDimension, sizeof (Dimension),
+	 XtOffset(MballWidget, core.height), XtRString, "400"},
+	{XtNmono, XtCMono, XtRBoolean, sizeof (Boolean),
+	 XtOffset(MballWidget, mball.mono), XtRString, "FALSE"},
+	{XtNreverse, XtCReverse, XtRBoolean, sizeof (Boolean),
+	 XtOffset(MballWidget, mball.reverse), XtRString, "FALSE"},
+	{XtNwedges, XtCWedges, XtRInt, sizeof (int),
+	 XtOffset(MballWidget, mball.wedges), XtRString, "8"},	/*DEFAULTWEDGES */
+	{XtNrings, XtCRings, XtRInt, sizeof (int),
+	 XtOffset(MballWidget, mball.rings), XtRString, "4"},	/*DEFAULTRINGS */
+	{XtNorient, XtCOrient, XtRBoolean, sizeof (Boolean),
+	 XtOffset(MballWidget, mball.orient), XtRString, "FALSE"},	/*DEFAULTORIENT */
+	{XtNpractice, XtCPractice, XtRBoolean, sizeof (Boolean),
+	 XtOffset(MballWidget, mball.practice), XtRString, "FALSE"},	/*DEFAULTPRACTICE */
+	{XtNbase, XtCBase, XtRInt, sizeof (int),
+	 XtOffset(MballWidget, mball.base), XtRString, "16"},
+	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
+	 XtOffset(MballWidget, mball.started), XtRString, "FALSE"},
+	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
+	 XtOffset(MballWidget, mball.select), XtRCallback, NULL}
 };
 
-	MballClassRec mballClassRec =
+MballClassRec mballClassRec =
+{
 	{
-		{
-			(WidgetClass) & widgetClassRec,		/* superclass */
-			"Mball",	/* class name */
-			sizeof (MballRec),	/* widget size */
-			NULL,	/* class initialize */
-			NULL,	/* class part initialize */
-			FALSE,	/* class inited */
-			(XtInitProc) InitializeMball,	/* initialize */
-			NULL,	/* initialize hook */
-			XtInheritRealize,	/* realize */
-			actionsListMball,	/* actions */
-			XtNumber(actionsListMball),	/* num actions */
-			resourcesMball,		/* resources */
-			XtNumber(resourcesMball),	/* num resources */
-			NULLQUARK,	/* xrm class */
-			TRUE,	/* compress motion */
-			TRUE,	/* compress exposure */
-			TRUE,	/* compress enterleave */
-			TRUE,	/* visible interest */
-			(XtWidgetProc) DestroyMball,	/* destroy */
-			(XtWidgetProc) ResizeMball,	/* resize */
-			(XtExposeProc) ExposeMball,	/* expose */
-			(XtSetValuesFunc) SetValuesMball,	/* set values */
-			NULL,	/* set values hook */
-			XtInheritSetValuesAlmost,	/* set values almost */
-			NULL,	/* get values hook */
-			NULL,	/* accept focus */
-			XtVersion,	/* version */
-			NULL,	/* callback private */
-			defaultTranslationsMball,	/* tm table */
-			NULL,	/* query geometry */
-			NULL,	/* display accelerator */
-			NULL	/* extension */
-		},
-		{
-			0	/* ignore */
-		}
+		(WidgetClass) & widgetClassRec,		/* superclass */
+		"Mball",	/* class name */
+		sizeof (MballRec),	/* widget size */
+		NULL,		/* class initialize */
+		NULL,		/* class part initialize */
+		FALSE,		/* class inited */
+		(XtInitProc) InitializeMball,	/* initialize */
+		NULL,		/* initialize hook */
+		XtInheritRealize,	/* realize */
+		actionsListMball,	/* actions */
+		XtNumber(actionsListMball),	/* num actions */
+		resourcesMball,	/* resources */
+		XtNumber(resourcesMball),	/* num resources */
+		NULLQUARK,	/* xrm class */
+		TRUE,		/* compress motion */
+		TRUE,		/* compress exposure */
+		TRUE,		/* compress enterleave */
+		TRUE,		/* visible interest */
+		(XtWidgetProc) DestroyMball,	/* destroy */
+		(XtWidgetProc) ResizeMball,	/* resize */
+		(XtExposeProc) ExposeMball,	/* expose */
+		(XtSetValuesFunc) SetValuesMball,	/* set values */
+		NULL,		/* set values hook */
+		XtInheritSetValuesAlmost,	/* set values almost */
+		NULL,		/* get values hook */
+		NULL,		/* accept focus */
+		XtVersion,	/* version */
+		NULL,		/* callback private */
+		defaultTranslationsMball,	/* tm table */
+		NULL,		/* query geometry */
+		NULL,		/* display accelerator */
+		NULL		/* extension */
+	},
+	{
+		0		/* ignore */
+	}
 };
 
 
 /* COORD == 12 not 8 */
-	static int  mapDirToWedge[(MAXWEDGES - MINWEDGES) / 2 + 1][12] =
+static int  mapDirToWedge[(MAXWEDGES - MINWEDGES) / 2 + 1][12] =
 {
 	{
 		0, 4, 4, 4, 0, 4, 4, 4
@@ -335,18 +348,16 @@ static int  mapWedgeToDir[(MAXWEDGES - MINWEDGES) / 2 + 1][12] =
 		0, 1, 2, 3, 4, 5, 6, 7, 8, 8, 8, 8
 	},
 	{
-		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 
+		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11
 	}
 };
 
 WidgetClass mballWidgetClass = (WidgetClass) & mballClassRec;
 
 static void
-InitializeMball(Widget request, Widget new)
+InitializeMball(Widget request, Widget renew)
 {
-	MballWidget w = (MballWidget) new;
-	XGCValues   values;
-	XtGCMask    valueMask;
+	MballWidget w = (MballWidget) renew;
 	int         wedge;
 
 	for (wedge = 0; wedge < MAXWEDGES; wedge++)
@@ -355,19 +366,8 @@ InitializeMball(Widget request, Widget new)
 	InitMoves();
 	ResetWedges(w);
 	(void) SRAND(getpid());
-	valueMask = GCForeground | GCBackground;
-	values.background = w->core.background_pixel;
-	values.foreground = w->mball.foreground;
-	w->mball.puzzleGC = XtGetGC(new, valueMask, &values);
-	values.foreground = w->mball.borderColor;
-	w->mball.borderGC = XtGetGC(new, valueMask, &values);
 	w->mball.depth = DefaultDepthOfScreen(XtScreen(w));
-	valueMask = GCForeground | GCBackground;
-	values.foreground = w->core.background_pixel;
-	values.background = w->mball.foreground;
-	w->mball.inverseGC = XtGetGC(new, valueMask, &values);
-	for (wedge = 0; wedge < MAXWEDGES; wedge++)
-		GetColor(w, wedge, TRUE);
+	SetAllColors(w, True);
 	ResizeMball(w);
 }
 
@@ -415,81 +415,57 @@ ResizeMball(MballWidget w)
 }
 
 static void
-ExposeMball(Widget new, XEvent * event, Region region)
-		 /* Not used */
+ExposeMball(Widget renew, XEvent * event, Region region)
 {
-	MballWidget w = (MballWidget) new;
+	MballWidget w = (MballWidget) renew;
 
 	if (w->core.visible) {
+		if (w->mball.reverse)
+			XFillRectangle(XtDisplay(w), XtWindow(w),
+				       w->mball.inverseGC, 0, 0, w->core.width, w->core.height);
 		DrawFrame(w, w->mball.puzzleGC);
 		DrawAllWedges(w);
 	}
 }
 
 static      Boolean
-SetValuesMball(Widget current, Widget request, Widget new)
+SetValuesMball(Widget current, Widget request, Widget renew)
 {
-	MballWidget c = (MballWidget) current, w = (MballWidget) new;
-	XGCValues   values;
-	XtGCMask    valueMask;
-	Boolean     redraw = FALSE;
+	MballWidget c = (MballWidget) current, w = (MballWidget) renew;
+	Boolean     redraw = False, setColors = False;
 	int         wedge;
 
 	CheckWedges(w);
-	if (w->mball.foreground != c->mball.foreground) {
-		valueMask = GCForeground | GCBackground;
-		values.foreground = w->mball.foreground;
-		values.background = w->core.background_pixel;
-		XtReleaseGC(new, w->mball.puzzleGC);
-		w->mball.puzzleGC = XtGetGC(new, valueMask, &values);
-		redraw = TRUE;
-	}
-	if (w->core.background_pixel != c->core.background_pixel) {
-		valueMask = GCForeground | GCBackground;
-		values.foreground = w->core.background_pixel;
-		values.background = w->mball.foreground;
-		XtReleaseGC(new, w->mball.inverseGC);
-		w->mball.inverseGC = XtGetGC(new, valueMask, &values);
-		redraw = TRUE;
-	}
-	if (w->mball.borderColor != c->mball.borderColor) {
-		valueMask = GCForeground | GCBackground;
-		values.foreground = w->mball.borderColor;
-		values.background = w->core.background_pixel;
-		XtReleaseGC(new, w->mball.borderGC);
-		w->mball.borderGC = XtGetGC(new, valueMask, &values);
-		redraw = TRUE;
-	}
-	if (w->mball.mono || w->mball.depth == 1) {
-		valueMask = GCForeground | GCBackground;
-		values.background = w->core.background_pixel;
-		values.foreground = w->mball.foreground;
-		for (wedge = 0; wedge < MAXWEDGES; wedge++) {
-			XtReleaseGC(new, w->mball.wedgeGC[wedge]);
-			w->mball.wedgeGC[wedge] = XtGetGC(new, valueMask, &values);
-		}
-		redraw = TRUE;
-	}
 	for (wedge = 0; wedge < MAXWEDGES; wedge++) {
-		if (strcmp(w->mball.wedgeName[wedge], c->mball.wedgeName[wedge]))
-			GetColor(w, wedge, FALSE);
+		if (strcmp(w->mball.wedgeName[wedge], c->mball.wedgeName[wedge])) {
+			setColors = True;
+			break;
+		}
 	}
-	if (w->mball.orient != c->mball.orient) {
+	if (w->core.background_pixel != c->core.background_pixel ||
+	    w->mball.foreground != c->mball.foreground ||
+	    w->mball.borderColor != c->mball.borderColor ||
+	    w->mball.reverse != c->mball.reverse ||
+	    w->mball.mono != c->mball.mono ||
+	    setColors) {
+		SetAllColors(w, False);
+		redraw = True;
+	}
+	if (w->mball.orient != c->mball.orient ||
+	    w->mball.base != c->mball.base ||
+	    w->mball.practice != c->mball.practice) {
 		ResetWedges(w);
-		redraw = TRUE;
-	} else if (w->mball.practice != c->mball.practice) {
-		ResetWedges(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	if (w->mball.wedges != c->mball.wedges ||
 	    w->mball.rings != c->mball.rings) {
 		ResetWedges(w);
 		ResizeMball(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	if (w->mball.mballLength != c->mball.mballLength) {
 		ResizeMball(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	return (redraw);
 }
@@ -699,7 +675,9 @@ UndoMball(MballWidget w, XEvent * event, char **args, int nArgs)
 static void
 SolveMball(MballWidget w, XEvent * event, char **args, int nArgs)
 {
-	/* SolveWedges(w); *//* Sorry, unimplemented */
+#if 0
+	SolveWedges(w);		/* Sorry, this is not implemented */
+#endif
 }
 
 static void
@@ -892,14 +870,66 @@ MoveMball(MballWidget w, int wedge, int ring, int direction, int control)
 }
 
 static void
-GetColor(MballWidget w, int wedge, int init)
+SetAllColors(MballWidget w, Boolean init)
+{
+	XGCValues   values;
+	XtGCMask    valueMask;
+	int         wedge;
+
+	valueMask = GCForeground | GCBackground;
+
+	if (w->mball.reverse) {
+		values.background = w->core.background_pixel;
+		values.foreground = w->mball.foreground;
+	} else {
+		values.foreground = w->core.background_pixel;
+		values.background = w->mball.foreground;
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->mball.inverseGC);
+	w->mball.inverseGC = XtGetGC((Widget) w, valueMask, &values);
+	if (w->mball.reverse) {
+		values.background = w->mball.foreground;
+		values.foreground = w->core.background_pixel;
+	} else {
+		values.foreground = w->mball.foreground;
+		values.background = w->core.background_pixel;
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->mball.puzzleGC);
+	w->mball.puzzleGC = XtGetGC((Widget) w, valueMask, &values);
+	if (w->mball.depth < 2 || w->mball.mono) {
+		if (w->mball.reverse) {
+			values.background = w->mball.foreground;
+			values.foreground = w->core.background_pixel;
+		} else {
+			values.foreground = w->mball.foreground;
+			values.background = w->core.background_pixel;
+		}
+	} else {
+		values.foreground = w->mball.borderColor;
+		values.background = w->core.background_pixel;
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->mball.borderGC);
+	w->mball.borderGC = XtGetGC((Widget) w, valueMask, &values);
+	for (wedge = 0; wedge < MAXWEDGES; wedge++)
+		GetColor(w, wedge, init);
+}
+
+static void
+GetColor(MballWidget w, int wedge, Boolean init)
 {
 	XGCValues   values;
 	XtGCMask    valueMask;
 	XColor      colorCell, rgb;
 
 	valueMask = GCForeground | GCBackground;
-	values.background = w->core.background_pixel;
+	if (w->mball.reverse) {
+		values.background = w->mball.foreground;
+	} else {
+		values.background = w->core.background_pixel;
+	}
 	if (w->mball.depth > 1 && !w->mball.mono) {
 		if (XAllocNamedColor(XtDisplay(w),
 				  DefaultColormap(XtDisplay(w), XtWindow(w)),
@@ -917,7 +947,13 @@ GetColor(MballWidget w, int wedge, int init)
 			XtWarning(buf);
 		}
 	}
-	values.foreground = w->mball.foreground;
+	if (w->mball.reverse) {
+		values.background = w->mball.foreground;
+		values.foreground = w->core.background_pixel;
+	} else {
+		values.background = w->core.background_pixel;
+		values.foreground = w->mball.foreground;
+	}
 	if (!init)
 		XtReleaseGC((Widget) w, w->mball.wedgeGC[wedge]);
 	w->mball.wedgeGC[wedge] = XtGetGC((Widget) w, valueMask, &values);
@@ -964,6 +1000,14 @@ CheckWedges(MballWidget w)
 		XtWarning(buf);
 		w->mball.rings = DEFAULTRINGS;
 	}
+	if (w->mball.base > 36) {
+		/* 10 numbers + 26 letters (ASCII or EBCDIC) */
+		XtWarning("Base must be less than or equal to 36");
+		w->mball.base = 16;	/* It has less digits for 12 wedges */
+	} else if (w->mball.base <= 1) {	/* Base 1 is rediculous :) */
+		XtWarning("Base must be greater than 1");
+		w->mball.base = 16;	/* It has less digits for 12 wedges */
+	}
 }
 
 static void
@@ -989,7 +1033,7 @@ ResetWedges(MballWidget w)
 			w->mball.mballLoc[wedge][ring].direction = DOWN;
 		}
 	FlushMoves(w);
-	w->mball.started = FALSE;
+	w->mball.started = False;
 }
 
 static void
@@ -1002,7 +1046,7 @@ ResizeWedges(MballWidget w)
 	w->mball.dr = w->mball.wedges;
 }
 
-static int
+static      Boolean
 SelectWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *view)
 {
 	double      angle, radius;
@@ -1037,18 +1081,18 @@ SelectWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *view)
 		*ring = w->mball.rings - 1 - *ring;
 	}
 	if (radius > w->mball.wedgeLength / 2 + w->mball.delta)
-		return FALSE;
-	return TRUE;
+		return False;
+	return True;
 }
 
-static int
+static      Boolean
 PositionWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *direction)
 {
 	int         view, inside;
 
 	inside = SelectWedges(w, x, y, wedge, ring, &view);
 	if ((*direction == CW || *direction == CCW) && !inside)
-		return FALSE;
+		return False;
 	if (view == DOWN) {
 		if (*direction == CCW)
 			*direction = CW;
@@ -1058,10 +1102,10 @@ PositionWedges(MballWidget w, int x, int y, int *wedge, int *ring, int *directio
 			*direction = (COORD - *direction) % COORD;
 	}
 	if (w->mball.wedges % 4 && (*direction == LEFT || *direction == RIGHT))
-		return FALSE;
+		return False;
 	if (w->mball.wedges <= 4 && *direction % 2 && *direction < COORD)
-		return FALSE;
-	return TRUE;
+		return False;
+	return True;
 }
 
 static void
@@ -1262,71 +1306,78 @@ OffsetSect(MballWidget w, int wedge, int *dx, int *dy)
 static void
 DrawSector(MballWidget w, int wedge, int ring, int offset)
 {
-	int         startx, starty, lengthx, lengthy;
+	GC          wedgeGC, borderGC;
+	int         startx, starty, lengthx, lengthy, i, l;
 
 	startx = 1 + w->mball.puzzleOffset.x;
 	starty = 1 + w->mball.puzzleOffset.y;
 	lengthx = w->mball.viewLength - w->mball.delta + w->mball.puzzleOffset.x;
 	lengthy = w->mball.viewLength - w->mball.delta + w->mball.puzzleOffset.y;
+	if (offset) {
+		borderGC = w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge];
+		if (w->mball.depth < 2 || w->mball.mono) {
+			wedgeGC = w->mball.inverseGC;
+		} else {
+			wedgeGC = w->mball.borderGC;
+		}
+	} else {
+		wedgeGC = w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge];
+		borderGC = w->mball.borderGC;
+	}
+	/* if the number of rings is odd
+	 * the ring can straddle both hemispheres */
 	if (ring < (w->mball.rings + 1) / 2)
-		if (offset)
-			DrawSect(w, w->mball.borderGC,
-				 w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge], ring, wedge,
-			 startx, starty, lengthx - startx, lengthy - starty);
-		else
-			DrawSect(w, w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge],
-				 w->mball.borderGC, ring, wedge,
+		DrawSect(w, wedgeGC, borderGC, ring, wedge,
 			 startx, starty, lengthx - startx, lengthy - starty);
 	if (ring + 1 > w->mball.rings / 2) {
 		if (w->mball.vertical)
-			if (offset)
-				DrawSect(w, w->mball.borderGC,
-					 w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge],
-					 w->mball.rings - 1 - ring,
-					 (3 * w->mball.wedges / 2 - 1 - wedge) % w->mball.wedges,
-					 startx, lengthy + 3, lengthx - startx, lengthy - starty);
-			else
-				DrawSect(w, w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge],
-				w->mball.borderGC, w->mball.rings - 1 - ring,
-					 (3 * w->mball.wedges / 2 - 1 - wedge) % w->mball.wedges,
-					 startx, lengthy + 3, lengthx - startx, lengthy - starty);
-		else if (offset)
-			DrawSect(w, w->mball.borderGC,
-				 w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge],
-				 w->mball.rings - 1 - ring, w->mball.wedges - 1 - wedge,
-				 lengthx + 3, starty, lengthx - startx, lengthy - starty);
+			DrawSect(w, wedgeGC, borderGC,
+				 w->mball.rings - 1 - ring,
+				 (3 * w->mball.wedges / 2 - 1 - wedge) % w->mball.wedges,
+				 startx, lengthy + 3, lengthx - startx, lengthy - starty);
 		else
-			DrawSect(w, w->mball.wedgeGC[w->mball.mballLoc[wedge][ring].wedge],
-				 w->mball.borderGC,
-				 w->mball.rings - 1 - ring, w->mball.wedges - 1 - wedge,
+			DrawSect(w, wedgeGC, borderGC,
+				 w->mball.rings - 1 - ring,
+				 w->mball.wedges - 1 - wedge,
 				 lengthx + 3, starty, lengthx - startx, lengthy - starty);
 	}
-	if (w->mball.depth == 1 || w->mball.mono) {
+	if (w->mball.depth < 2 || w->mball.mono) {
 		int         letterX, letterY;
 		char        buf[3];
 
+		if (offset) {
+			borderGC = w->mball.borderGC;
+		} else {
+			borderGC = w->mball.inverseGC;
+		}
 		if (ring < (w->mball.rings + 1) / 2) {
 			LetterPosition(w, wedge, ring, lengthx - startx, lengthy - starty,
 				       &letterX, &letterY);
 			letterX += startx + w->mball.letterOffset.x;
 			letterY += starty + w->mball.letterOffset.y;
 			if (w->mball.orient && !w->mball.mballLoc[wedge][ring].direction) {
-				if (w->mball.mballLoc[wedge][ring].wedge < 10)
-				(void) sprintf(buf, "%d%c", w->mball.mballLoc[wedge][ring].wedge,
-					       w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0]);
-				else
-				(void) sprintf(buf, "%c%c",
-					 w->mball.mballLoc[wedge][ring].wedge + 'a' - 10,
-					       w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0]);
-				letterX += w->mball.letterOffset.x;
-				XDrawString(XtDisplay(w), XtWindow(w), w->mball.inverseGC,
-					    letterX, letterY, buf, 2);
+				int         last;
+
+				l = w->mball.mballLoc[wedge][ring].wedge;
+				last = int2String(buf, l, w->mball.base, False);
+				buf[last] = w->mball.wedgeName[l][0];
+				buf[last + 1] = '\0';
+				i = 0;
+				if (l == 0)
+					l = 1;
+				l *= w->mball.base;
+				while (l >= 1) {
+					l /= w->mball.base;
+					letterX += w->mball.letterOffset.x;
+					i++;
+				}
 			} else {
-				(void) sprintf(buf, "%c",
-					       w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0]);
-				XDrawString(XtDisplay(w), XtWindow(w), w->mball.inverseGC,
-					    letterX, letterY, buf, 1);
+				buf[0] = w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0];
+				buf[1] = '\0';
+				i = 1;
 			}
+			XDrawString(XtDisplay(w), XtWindow(w), borderGC,
+				    letterX, letterY, buf, i);
 		}
 		if (ring + 1 > w->mball.rings / 2) {
 			if (w->mball.vertical) {
@@ -1347,22 +1398,28 @@ DrawSector(MballWidget w, int wedge, int ring, int offset)
 				letterY += starty + w->mball.letterOffset.y;
 			}
 			if (w->mball.orient && w->mball.mballLoc[wedge][ring].direction) {
-				if (w->mball.mballLoc[wedge][ring].wedge < 10)
-				(void) sprintf(buf, "%d%c", w->mball.mballLoc[wedge][ring].wedge,
-					       w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0]);
-				else
-				(void) sprintf(buf, "%c%c",
-					 w->mball.mballLoc[wedge][ring].wedge + 'a' - 10,
-					 w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0]);
-				letterX += w->mball.letterOffset.x;
-				XDrawString(XtDisplay(w), XtWindow(w), w->mball.inverseGC,
-					    letterX, letterY, buf, 2);
+				int         last;
+
+				l = w->mball.mballLoc[wedge][ring].wedge;
+				last = int2String(buf, l, w->mball.base, False);
+				buf[last] = w->mball.wedgeName[l][0];
+				buf[last + 1] = '\0';
+				i = 0;
+				if (l == 0)
+					l = 1;
+				l *= w->mball.base;
+				while (l >= 1) {
+					l /= w->mball.base;
+					letterX += w->mball.letterOffset.x;
+					i++;
+				}
 			} else {
-				(void) sprintf(buf, "%c",
-					       w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0]);
-				XDrawString(XtDisplay(w), XtWindow(w), w->mball.inverseGC,
-					    letterX, letterY, buf, 1);
+				buf[0] = w->mball.wedgeName[w->mball.mballLoc[wedge][ring].wedge][0];
+				buf[1] = '\0';
+				i = 1;
 			}
+			XDrawString(XtDisplay(w), XtWindow(w), borderGC,
+				    letterX, letterY, buf, i);
 		}
 	} else if (w->mball.orient) {
 		int         letterX, letterY;
@@ -1374,9 +1431,18 @@ DrawSector(MballWidget w, int wedge, int ring, int offset)
 				       &letterX, &letterY);
 			letterX += startx + w->mball.letterOffset.x;
 			letterY += starty + w->mball.letterOffset.y;
-			(void) sprintf(buf, "%d", w->mball.mballLoc[wedge][ring].wedge);
-			XDrawString(XtDisplay(w), XtWindow(w), w->mball.inverseGC,
-				    letterX, letterY, buf, 1);
+			l = w->mball.mballLoc[wedge][ring].wedge;
+			(void) int2String(buf, l, w->mball.base, True);
+			i = 0;
+			if (l == 0)
+				l = 1;
+			while (l >= 1) {
+				l /= w->mball.base;
+				letterX += w->mball.letterOffset.x;
+				i++;
+			}
+			XDrawString(XtDisplay(w), XtWindow(w), borderGC,
+				    letterX, letterY, buf, i);
 		}
 		if (ring + 1 > w->mball.rings / 2 &&
 		    w->mball.mballLoc[wedge][ring].direction) {
@@ -1397,9 +1463,18 @@ DrawSector(MballWidget w, int wedge, int ring, int offset)
 				letterX += lengthx + 3 + w->mball.letterOffset.x;
 				letterY += starty + w->mball.letterOffset.y;
 			}
-			(void) sprintf(buf, "%d", w->mball.mballLoc[wedge][ring].wedge);
-			XDrawString(XtDisplay(w), XtWindow(w), w->mball.inverseGC,
-				    letterX, letterY, buf, 1);
+			l = w->mball.mballLoc[wedge][ring].wedge;
+			(void) int2String(buf, l, w->mball.base, True);
+			i = 0;
+			if (l == 0)
+				l = 1;
+			while (l >= 1) {
+				l /= w->mball.base;
+				letterX += w->mball.letterOffset.x;
+				i++;
+			}
+			XDrawString(XtDisplay(w), XtWindow(w), borderGC,
+				    letterX, letterY, buf, i);
 		}
 	}
 }
@@ -1558,28 +1633,70 @@ XDrawSector(Display * display, Drawable drawable, GC gc, int xo, int yo, int wid
 	   (int) ((double) r2 * x) + xo, (int) ((double) r2 * y) + yo); */
 }
 
-/*-
-static void XFillSector(display, drawable, gc,
-    xo, yo, width1, height1, width2, height2, angle1, angle2)
-  Display *display;
-  Drawable drawable;
-  GC gc;
-  int xo, yo;
-  int width1, height1, width2, height2;
-  int angle1, angle2;
+#if 0
+static void
+XFillSector(display, drawable, gc,
+	    xo, yo, width1, height1, width2, height2, angle1, angle2)
+	Display    *display;
+	Drawable    drawable;
+	GC          gc;
+	int         xo, yo;
+	int         width1, height1, width2, height2;
+	int         angle1, angle2;
 {
-  int d, r1 = MIN(width1, height1) / 2, r2 = MIN(width2, height2) / 2;
+	int         d, r1 = MIN(width1, height1) / 2, r2 = MIN(width2, height2) / 2;
 
-  if (r1 > r2) {
-    d = r1; r1 = r2; r2 = d;
-  }
-  if (r1 < 0)
-    r1 = -3;
-  for (d = 2 * (r1 + 3); d < 2 * (r2 - 1); d++)
-    XDrawArc(display, drawable, gc, xo - d / 2, yo - d / 2, d, d,
-      angle1, angle2);
+	if (r1 > r2) {
+		d = r1;
+		r1 = r2;
+		r2 = d;
+	}
+	if (r1 < 0)
+		r1 = -3;
+	for (d = 2 * (r1 + 3); d < 2 * (r2 - 1); d++)
+		XDrawArc(display, drawable, gc, xo - d / 2, yo - d / 2, d, d,
+			 angle1, angle2);
+}
+#endif
+
+static int
+int2String(char *buf, int number, int base, Boolean capital)
+{
+	int         digit, mult = base, last, position;
+	int         a, i, j, s, r;
+
+	if (capital) {
+		a = 'A', i = 'I', j = 'J', s = 'S', r = 'R';
+	} else {
+		a = 'a', i = 'i', j = 'j', s = 's', r = 'r';
+	}
+	if (number < 0) {
+		(void) printf("number %d < 0\n", number);
+		return 0;
+	}
+	last = 1;
+	while (number >= mult) {
+		last++;
+		mult *= base;
+	}
+	for (position = 0; position < last; position++) {
+		mult /= base;
+		digit = number / mult;
+		number -= digit * mult;
+		buf[position] = digit + '0';
+		if (buf[position] > '9') {	/* ASCII */
+			buf[position] += (a - '9' - 1);
+		} else if (buf[position] < '0') {	/* EBCDIC */
+			buf[position] += (a - '9' - 1);
+			if (buf[position] > i)
+				buf[position] += (j - i - 1);
+			if (buf[position] > r)
+				buf[position] += (s - r - 1);
+		}
+	}
+	buf[last] = '\0';
+	return last;
 }
-*/
 Boolean
 CheckSolved(MballWidget w)
 {
@@ -1594,15 +1711,15 @@ CheckSolved(MballWidget w)
 			}
 			for (ring = 0; ring < w->mball.rings; ring++) {
 				if (test.direction != w->mball.mballLoc[wedge][ring].direction)
-					return FALSE;
+					return False;
 				if (test.direction) {
 					if ((w->mball.wedges - w->mball.mballLoc[wedge][ring].wedge +
 					     test.wedge) % w->mball.wedges != wedge)
-						return FALSE;
+						return False;
 				} else {
 					if ((w->mball.wedges + w->mball.mballLoc[wedge][ring].wedge -
 					     test.wedge) % w->mball.wedges != wedge)
-						return FALSE;
+						return False;
 				}
 			}
 	} else
@@ -1612,8 +1729,8 @@ CheckSolved(MballWidget w)
 					test.wedge = w->mball.mballLoc[wedge][ring].wedge;
 					test.direction = w->mball.mballLoc[wedge][ring].direction;
 				} else if (test.wedge != w->mball.mballLoc[wedge][ring].wedge)
-					return FALSE;
-	return TRUE;
+					return False;
+	return True;
 }
 
 #ifdef DEBUG
